# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2021-03-03 15:13
from __future__ import unicode_literals

from django.db import migrations, models

from dj_dynamic_settings.compat import JSONField
import dj_dynamic_settings.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Setting',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(max_length=128, unique=True, validators=[dj_dynamic_settings.models.registry_contains_validator])),
                ('value', JSONField()),
                ('is_active', models.BooleanField(default=True)),
                ('created_date', models.DateTimeField(auto_now_add=True)),
                ('modified_date', models.DateTimeField(auto_now=True)),
            ],
        ),
    ]
